/****************************************************************************
 * buildtracker.h
 *  Copyright © 2015, Nicolay Izoderov <nico-izo@ya.ru>.
 *  Licence: GPLv3 or later
 *
 ****************************************************************************
 *                                                                          *
 *   This library is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 3 of the License, or      *
 *   (at your option) any later version.                                    *
 *                                                                          *
 ****************************************************************************/

#ifndef BUILDTRACKER_H
#define BUILDTRACKER_H

#include <QObject>
#include <QNetworkAccessManager>
#include <QTimer>
#include <QHash>

class Shortener;

struct BuildInfo {
	/**
	 * Datetime formatted as yyyy-MM-ddTHH:mm:ss.zzzQQQ+00:00, where QQQ is microseconds
	 */
	QString datetime;
	/**
	 * \see https://launchpad.net/+apidoc/1.0.html#build
	 */
	QString webLink;
	/**
	 * Can be "Successfully built", or "Failed to build", or many others
	 * \see https://launchpad.net/+apidoc/1.0.html#build
	 */
	QString status;

	QString toString(Shortener *shortener) const;
};

class BuildTracker : public QObject
{
	Q_OBJECT

public:
	explicit BuildTracker(Shortener *shortener, QObject *parent = 0);
	~BuildTracker();

signals:
	void newBuild(const QString&, const QString&);
public slots:
	QString getRepoMessage(const QString &repo);
private slots:
	void updateRepos();
	void onReplyFinished(QNetworkReply*);
private:
	QTimer updater;
	QNetworkAccessManager *netman;
	QHash<QString, QString> repos;
	Shortener *shortener;

	QHash<QString, QDateTime> lastUpdated;
	QHash<QString, QList<BuildInfo>> builds;
	QList<BuildInfo> jsonToInfoList(const QByteArray &jsonstr);
};

#endif // BUILDTRACKER_H
