/****************************************************************************
 * settings.h
 *  Copyright © 2012, Vsevolod Velichko <torkvema@gmail.com>.
 *  Licence: GPLv3 or later
 *
 ****************************************************************************
 *                                                                          *
 *   This library is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 3 of the License, or      *
 *   (at your option) any later version.                                    *
 *                                                                          *
 ****************************************************************************/

#ifndef SETTINGS_Z7VQXH4J
#define SETTINGS_Z7VQXH4J

#include <QSharedPointer>
#include <QSettings>
#include <QMetaType>
#include <QHash>

typedef QHash<QString,QString> NICKHASH;
Q_DECLARE_METATYPE(NICKHASH)

class Settings
{
	private:
		QSharedPointer<QSettings> settings;
	public:
		Settings();

		template <typename T> T get(const QString &key) const
		{
			const QVariant &v = settings->value(key);
			if (v.canConvert<T>())
				return v.value<T>();
			return T();
		}
		template <typename T> void set(const QString &key, const T& value)
		{
			settings->setValue(key, QVariant::fromValue(value));
		}

		const static Settings& instance();
		static Settings& mutableInstance();
};

#endif /* end of include guard: SETTINGS_Z7VQXH4J */

