/****************************************************************************
 * shortener.cpp
 *  Copyright © 2011,2015, Vsevolod Velichko.
 *  Licence: GPLv3 or later
 *
 ****************************************************************************
 *                                                                          *
 *   This library is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 3 of the License, or      *
 *   (at your option) any later version.                                    *
 *                                                                          *
 ****************************************************************************/

#include "shortener.h"
#include <QDebug>
#include <QNetworkRequest>
#include <QNetworkReply>
#include <QScopedPointer>
#include <QEventLoop>
#include <QUrlQuery>

QString Shortener::shorten(const QString &/* url */)
{
	return QString();
}
QString Shortener::expand(const QString &/* url */)
{
	return QString();
}
Shortener::~Shortener()
{
}

Bitly::Bitly(const QString &user, const QString &key)
{
	setUsername(user);
	setApikey(key);
}

Bitly::~Bitly()
{
}

void Bitly::setUsername(const QString &user)
{
	if (user.isEmpty())
		qDebug() << "Bitly: username is empty, service will be unavailable";
	username = user;
}

void Bitly::setApikey(const QString &key)
{
	if (key.isEmpty())
		qDebug() << "Bitly: apikey is empty, service will be unavailable";
	apikey = key;
}

QString Bitly::shorten(const QString &url)
{
	if (username.isEmpty() || apikey.isEmpty())
	{
		qDebug() << "Bitly: you has been warned, bit.ly api is unavailable to you";
		return QString();
	}
	QUrl request("http://api.bit.ly/v3/shorten");
	QUrlQuery query;
	query.addQueryItem("longUrl", url);
	query.addQueryItem("login", username);
	query.addQueryItem("apiKey", apikey);
	query.addQueryItem("format", "txt");
	request.setQuery(query);

	return fetchUrl(request);

}

QString Bitly::expand(const QString &url)
{
	if (username.isEmpty() || apikey.isEmpty())
	{
		qDebug() << "Bitly: you has been warned, bit.ly api is unavailable to you";
		return QString();
	}
	QUrl request("http://api.bit.ly/v3/expand");
	QUrlQuery query;
	query.addQueryItem("shortUrl", url);
	query.addQueryItem("login", username);
	query.addQueryItem("apiKey", apikey);
	query.addQueryItem("format", "txt");
	request.setQuery(query);

	return fetchUrl(request);
}

QString Bitly::fetchUrl(const QUrl &url)
{
	qDebug() << url.toString();
	QEventLoop loop;
	QNetworkRequest req(url);
	QScopedPointer<QNetworkReply> reply(qnam.get(req));
	QObject::connect(reply.data(), SIGNAL(finished()), &loop, SLOT(quit()));
	loop.exec();
	if (reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt() != 200)
	{
		qDebug() << "Bitly: got no reply";
		return QString();
	}
	return QString::fromUtf8(reply->readAll().constData());
}


Clckru::Clckru()
{
}

Clckru::~Clckru()
{
}

QString Clckru::shorten(const QString &url)
{
	QUrl request("https://clck.ru/--?url=" + QUrl::toPercentEncoding(url));

	return fetchUrl(request);
}

QString Clckru::expand(const QString &url)
{
	Q_UNUSED(url);
	qDebug() << Q_FUNC_INFO << "Not implemented";
	return QString();
}

QString Clckru::fetchUrl(const QUrl &url)
{
	QEventLoop loop;
	qDebug() << url;

	QNetworkRequest req(url);
	QScopedPointer<QNetworkReply> reply(qnam.get(req));
	QObject::connect(reply.data(), SIGNAL(finished()), &loop, SLOT(quit()));
	loop.exec();
	if (reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt() != 200)
	{
		qDebug() << reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
		qDebug() << reply->errorString();
		qDebug() << "Clckru: got no reply";
		return QString();
	}
	return QString::fromUtf8(reply->readAll().constData());
}
