/****************************************************************************
 * shortener.h
 *  Copyright © 2011,2015, Vsevolod Velichko.
 *  Licence: GPLv3 or later
 *
 ****************************************************************************
 *                                                                          *
 *   This library is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 3 of the License, or      *
 *   (at your option) any later version.                                    *
 *                                                                          *
 ****************************************************************************/

#ifndef BITLY_RFWF4A3Y
#define BITLY_RFWF4A3Y

#include <QString>
#include <QUrl>
#include <QNetworkAccessManager>


class Shortener
{
	public:
		virtual QString shorten(const QString &url) = 0;
		virtual QString expand(const QString &url) = 0;
		virtual ~Shortener() = 0;
};

/** 
 * @brief Bit.ly service connector.
 * Provides you easy way to shorten or expand your URLs.
 */
class Bitly : public Shortener
{
	public:
		Bitly(const QString &user, const QString &key);
		virtual ~Bitly();

		void setUsername(const QString &user);
		void setApikey(const QString &key);

		virtual QString shorten(const QString &url);
		virtual QString expand(const QString &url);
	private:
		QString fetchUrl(const QUrl &url);

		QString username;
		QString apikey;
		QNetworkAccessManager qnam;
};

class Clckru : public Shortener
{
	Q_DISABLE_COPY(Clckru)
	public:
		Clckru();
		virtual ~Clckru();

		virtual QString shorten(const QString &url);
		virtual QString expand(const QString &url);
	private:
		QString fetchUrl(const QUrl &url);
		QNetworkAccessManager qnam;
};

#endif /* end of include guard: BITLY_RFWF4A3Y */
